
To complete this challenge, you need to know some principles

## Principles

### Principle - Styles
As of Sep 2024, there are two styles of tab bars: Android and iOS. For customizing your own tab bar style, that will be mentioned later

Android tab bar: You can see that the Android tab bar is at the top. It follows Material UI Design principles. The active tab is visually connected to the opened tab via the blue streak. The user expects to be able to swipe gesture between tabs. 

![](0mDgWLX.png)


iOS tab bar: You can see that the iOS tab bar is at the bottom. It follows Apple design principles. The user CANNOT swipe gesture between tabs. 
![](f3S2aQw.png)


FYI on the buttons: If you noticed the buttons, they are also different between Android and iOS but React Native takes care of that automatically for the `<Button>` imported from React Native.

You will need to have different files for android/ios/web or dynamically load the tab bar based on Platform (`Platform.OS==="iOS`, etc), OR stick to one style. It is against recommendations to stick to one style because you don't want to jar the user with a different OS style.

To dynamically load the tab bar:
```
import { Platform } from 'react-native';

// ...

const TabBarConfig = getProperDesignForTabBar();
function getProperDesignForTabBar() {
  switch(Platform.OS) {
    case "android": 
      return createMaterialTopTabNavigator();
      break;
    case "ios": 
      return createBottomTabNavigator();
      break;
    default: // web
      return createBottomTabNavigator();
  }
}
```

However, you could customize your own style of tab bar, and we will cover that in the next section.

Optionally, refer to [[React Native Navigation - Tab bars on iOS vs Android]]

### Principle - Custom Style

The `react-native-tab-view` package lets you create a cross platform tab bar. If you don't customize it by passing your own custom tabbar component with `<TabView renderTabBar={props => <TabBar {...props} />}`, then it would default to an Android Material UI tab bar.

For how to implement custom style, refer to https://reactnavigation.org/docs/tab-view/

As with our challenge, we will dynamically load the proper styled tab bar based on Platform.OS instead of creating a custom tab bar.

### Principle - Syntax of iOS or Android styles

After you load the tab bar, with either:

- `const TabBarConfig = createMaterialTopTabNavigator();`
- `const TabBarConfig = createBottomTabNavigator();`
- Or you load dynamically based on `Platform.OS`, which is covered above.

The syntax is the same when configuring the tabs the user sees and the screen each tab opens, so doesn't matter which Tab Navigator creator was called:
```
function TabNavigator() {

  return (
    <TabBarConfig.Navigator
      initialRouteName="TabOne"
      screenOptions={{ tabBarActiveTintColor: "#2f95dc" }}
    >
      <TabBarConfig.Screen
        name="TabOne"
        component={TabOneComponent}
        options={{
          headerShown: false,
          tabBarIcon: ({ color }) => (
            <Ionicons name="code" color={color} size={30} style={{ marginBottom: -3 }} />
          ),
        }}
      />
      <TabBarConfig.Screen
        name="TabTwo"
        component={TabTwoComponent}
        options={{
          headerShown: false,
          tabBarIcon: ({ color }) => (
            <Ionicons name="code" color={color} size={30} style={{ marginBottom: -3 }} />
          ),
        }}
      />
    </TabBarConfig.Navigator>
  );
}
```
^ Where the component `<TabNavigator>` is rendered on some parent component. The active tab will automatically be the first screen configured in Tab Navigator.

Note you are not required to call the configurable component returned by the tab bar creator as "TabBarConfig".  You can name it something else if you would like.

### Principle - Named Routes

- Each screen has a unique name given to it in the Tab Navigator syntax (`<Var.Screen name=”...”...>`). They are still named routes.
	- Proof 1: You can choose another tab besides the first tab to render first when the Tab Navigator renders. You do this via the prop `initialRouteName`  in `Var.Navigator` .
	- Proof 2: You can use `Pressable` or equivalent (touchable, button) inside any of the screens opened from the tab bar, and its onPress can have `useNavigation`'s `var.navigate`  to navigate to another tab by its named route.

### Principle - NOT a Stack Navigator
Even though creating Tab Navigator with its tabs and corresponding screens looks like a similar syntax to Stack Navigator (For either case: Var.Navigator>Var.Screen\*N), the Tab Navigator is not a **Stack** Navigator which remembers the order in which you opened the screens - they just both happen to involve navigation and screens. There's no swiping back to go to a previous screen with a swiping animation displayed. But when it comes to swiping, if you're using Android's material ui tab bar, you can swipe back and forth to switch tabs, but that is not a stack (popping off a screen to go to the previous screen).

### Principe - How it renders

Tab Navigator component renders the tab bar AND the first screen (unless specified a different initial screen via the initialRouteName prop in Var.Navigator). Think of the Tab Navigator as actually a panel attached to the tab bar.


---

## Challenge

You can use [with-tab-navigation](https://github.com/expo/examples/tree/master/with-tab-navigation "with-tab-navigation") or build up from the previous challenge (recommended). The answer for comparing or giving up is based on you following the previous challenge. If you choose to create a blank slate, you can use either `--example with-tab-navigation` or `--example navigation`. The `--example navigation` is what the previous string of challenges that build on each other is based on: this boilerplate includes both Tab Navigator and Stack Navigator.

Your goal is to create the two screenshots above. When opening iOS, it shows the iOS styled tab bar. When opening Android, it shows the Android styled tab bar. It doesn't matter if virtual device or physical device. Opening web on your computer will show the iOS or Android, whichever you preferred.

All the previous challenges' capabilities like entering user information at "Screen Two", then going to "Screen Two Details", that next screen shows the user information that was entered from the previous screen.

---


## Answer

You install the two styles of tab bars in the active project:
- @react-navigation/bottom-tabs
- @react-navigation/material-top-tabs

For simplicity, the code is all flattened in App.js:
```
import "react-native-gesture-handler";
import { SafeAreaProvider } from "react-native-safe-area-context";
import { View, Text, TouchableOpacity, Platform } from 'react-native';
import { NavigationContainer } from '@react-navigation/native';
import { createStackNavigator } from "@react-navigation/stack";
import TabOneScreen from './screens/TabOneScreen'
import TabTwoScreen from './screens/TabTwoScreen'
import TabTwoDetailsScreen from './screens/TabTwoDetailsScreen'

import { createBottomTabNavigator } from "@react-navigation/bottom-tabs";
import { createMaterialTopTabNavigator } from "@react-navigation/material-top-tabs";

import Ionicons from "@expo/vector-icons/Ionicons";

const RootStackConfig = createStackNavigator();
export default function App() {
  return (
    <SafeAreaProvider>
      <NavigationContainer>
        <RootStackConfig.Navigator screenOptions={{ headerShown: false }}>
          <RootStackConfig.Screen name="Root" component={TabNavigator} />
        </RootStackConfig.Navigator>
      </NavigationContainer>
    </SafeAreaProvider>
  );
}

const TabBarConfig = getProperDesignForTabBar();
function getProperDesignForTabBar() {
  switch(Platform.OS) {
    case "android": 
      return createMaterialTopTabNavigator();
      break;
    case "ios": 
      return createBottomTabNavigator();
      break;
    default: // web
      return createBottomTabNavigator();
  }
}

function TabNavigator() {

  return (
    <TabBarConfig.Navigator
      initialRouteName="TabOne"
      screenOptions={{ tabBarActiveTintColor: "#2f95dc" }}
    >
      <TabBarConfig.Screen
        name="TabOne"
        component={TabOneComponent}
        options={{
          headerShown: false,
          tabBarIcon: ({ color }) => (
            <Ionicons name="code" color={color} size={30} style={{ marginBottom: -3 }} />
          ),
        }}
      />
      <TabBarConfig.Screen
        name="TabTwo"
        component={TabTwoComponent}
        options={{
          headerShown: false,
          tabBarIcon: ({ color }) => (
            <Ionicons name="code" color={color} size={30} style={{ marginBottom: -3 }} />
          ),
        }}
      />
    </TabBarConfig.Navigator>
  );
}

const TabOneStackConfig = createStackNavigator();
const TabOneComponent = () => {
  return (
    <TabOneStackConfig.Navigator>
    <TabOneStackConfig.Screen
      name="TabOneScreen"
      component={TabOneScreen}
      options={{ headerTitle: "TabOne from tab bar" }}
    />
    </TabOneStackConfig.Navigator>
  )
}

const TabTwoStackConfig = createStackNavigator();
function TabTwoComponent() {
  return (
    <TabTwoStackConfig.Navigator>
      <TabTwoStackConfig.Screen
        name="TabTwoScreen"
        component={TabTwoPanel}
        options={{ headerTitle: "TabTwo from tab bar" }}
      />
    </TabTwoStackConfig.Navigator>
  );
}


const TabTwoPanelStackConfig = createStackNavigator();
function TabTwoPanel() {
  return (
      <TabTwoPanelStackConfig.Navigator screenOptions={{ headerShown: true }}>
        <TabTwoPanelStackConfig.Screen name="TabTwo" component={TabTwoScreen}/>
        <TabTwoPanelStackConfig.Screen name="TabTwoDetails" component={TabTwoDetailsScreen} />
        <TabTwoPanelStackConfig.Screen name="TabOne" component={TabOneScreen}/>
      </TabTwoPanelStackConfig.Navigator>
  );
}
```

For simplicity, the code is all flattened in App.js... You are welcomed to refactor them into separate files. Recommendations: 
- RootStackConfig() remains at App.js and can be renamed as App()
- A separate file can contain the tab bar and each tab component: TabNavigator, TabOneComponent, TabTwoComponent. File could be named to allude to the toolbar.
- A separate file can contain the Stack Navigator that can track "Screen One", "Screen Two", and "Screen Two Details". File could be named to allude to a panel because it's the panel attached to the tab bar.
